var { ToggleButton } = require( 'sdk/ui/button/toggle' );
var panels = require( 'sdk/panel' );
var self = require( 'sdk/self' );
var tabs = require( 'sdk/tabs' );
var windows = require( 'sdk/windows' ).browserWindows;
var preferences = require( 'sdk/simple-prefs' );
var request = require( 'sdk/request' ).Request;
var simpleStorage = require( 'sdk/simple-storage' );
var selection = require( 'sdk/selection' );
var pageMod = require( 'sdk/page-mod' );
var selectedText, deviceID;

pageMod.PageMod( {
  include: "*",
  contentScriptFile: self.data.url( 'page.js' ),
  contentScriptOptions: {
    simpleStorage: simpleStorage,
    icons: [ self.data.url( 'smartphone-24.png' ), self.data.url( 'tablet-24.png' ), self.data.url( 'computer-24.png' ), self.data.url( 'phablet-24.png' ) ],
    showIcons: preferences.prefs['displayType'] == 0 || preferences.prefs['displayType'] == 1,
    showLabel: preferences.prefs['displayType'] == 0 || preferences.prefs['displayType'] == 2,
    showButtons: preferences.prefs['preFormat'] != 2,
    withKey: ( preferences.prefs['preFormat'] == 1 ) ? preferences.prefs['customKey'] : -1
  },
  onAttach: function( worker ) {
    worker.port.on( 'click', function( id, links ) {
      var name;
  
      deviceID = id;
      selectedText = links;
      if ( preferences.prefs['nameType'] == 0 ) {
	package.show();
      } else {
	if ( preferences.prefs['nameType'] == 1 ) {
	  name = tabs.activeTab.title;
	} else if ( preferences.prefs['nameType'] == 2 ) {
	  var date;
	  
	  date = new Date();
	  name = date.toLocaleString();
	}
	if ( name.length > 100 ) {
	  name = name.substr( 0, 99 );
	}
	sendLinks( name );
      }	      
    });
  }
} );

preferences.on( 'refresh', function() {
  var key = preferences.prefs['key'];
  var username = preferences.prefs['username'];
  if ( key == '' ) {
    console.log( 'You must provide a Web API Key first!' );
  } else if ( username == '' ) {
    console.log( 'You must provide your username!' );
  } else {
    var refreshRequest = request( {
      url: "http://www.vincescodes.com/jdcaptcha/webapi.php?username=" + username + "&web_password=" + key + "&action=devices&version=" + self.version,
      onComplete: function ( response ) {
	if ( response.status == 200 ) {
          var status;

          status = response.json;
          if ( status.status == 'OK' ) {
	    simpleStorage.storage.devices = status.devices;
	  } else {
	    console.log( status.status + ': ' + status.msg );
	  }
	} else {
          console.log( response.status + ': ' + response.statusText );
	}
      }
    } );
    refreshRequest.get();
  }
} );

var button = ToggleButton( {
  id: "jdcaptcha-button",
  label: "jdCaptcha",
  icon: {
    "16": "./logo-16.png",
    "32": "./logo-32.png",
    "64": "./logo-64.png"
  },
  onChange: handleChange
} );

var panel = panels.Panel( {
  contentURL: self.data.url( 'panel.html' ),
  contentScriptFile: self.data.url( 'panel.js' ),
  height: 470,
  onHide: handleHide,
  width: 300
} );

var package = panels.Panel( {
  contentURL: self.data.url( 'package.html' ),
  contentScriptFile: self.data.url( 'package.js' ),
  height: 50,
  width: 250
} );

panel.port.on( 'click-link', function( url ) {
  tabs.open( url );
  panel.hide();
} );

package.on( 'show', function() {
  package.port.emit( 'show' );
} );

package.port.on( 'text-entered', function( text ) {
  if ( text.length > 100 ) {
    text = text.substr( 0, 99 );
  }
  sendLinks( text );
  package.hide();
} );

panel.port.on( 'click-device', function( id ) {
  var name;
  
  panel.hide();
  deviceID = id;
  if ( preferences.prefs['nameType'] == 0 ) {
    package.show();
  } else {
    if ( preferences.prefs['nameType'] == 1 ) {
      name = tabs.activeTab.title;
    } else if ( preferences.prefs['nameType'] == 2 ) {
      var date;
      
      date = new Date();
      name = date.toLocaleString();
    }
    if ( name.length > 100 ) {
      name = name.substr( 0, 99 );
    }
    sendLinks( name );
  }
} );

function sendLinks( name ) {
  sendRequest = request( {
    url: "http://www.vincescodes.com/jdcaptcha/webapi.php",
    content: {
      "username": preferences.prefs['username'],
      "web_password": preferences.prefs['key'],
      "version": self.version,
      "action": "send",
      "id": deviceID,
      "name": name,
      "links": selectedText
    },
    onComplete: function ( response ) {
      if ( response.status == 200 ) {
        var status;

        status = response.json;
        if ( status.status != 'OK' ) {
	  console.log( status.status + ': ' + status.msg );
	}
      } else {
        console.log( response.status + ': ' + response.statusText );
      }
    }
  } );
  sendRequest.post();
}

panel.on( 'show' , function() {
  selectedText = false;
  if ( selection.text ) {
    selectedText = '';
    if ( !selection.isContiguous ) {
      for ( var subselection in selection ) {
	selectedText += subselection.text;
      }
    } else {
      selectedText = selection.text;
    }
  }
  panel.port.emit( 'show', simpleStorage, selectedText );
} );

function handleChange( state ) {
  if ( state.checked ) {
    panel.show( {
      position: button
    } );
  }
}

function handleHide() {
  button.state( 'window', {checked: false} );
}

